@@ -15,7 +15,6 @@ from sanic.exceptions import (
 )
 from sanic.request import Request
 from sanic.response import json, text
-from sanic.views import CompositionView
 
 
 # ------------------------------------------------------------ #
@@ -83,7 +82,6 @@ def test_versioned_routes_get(app, method):
             return text("OK")
 
     else:
-        print(func)
         raise Exception(f"{func} is not callable")
 
     app.blueprint(bp)
@@ -477,6 +475,58 @@ def test_bp_exception_handler(app):
     assert response.status == 200
 
 
+def test_bp_exception_handler_applied(app):
+    class Error(Exception):
+        pass
+
+    handled = Blueprint("handled")
+    nothandled = Blueprint("nothandled")
+
+    @handled.exception(Error)
+    def handle_error(req, e):
+        return text("handled {}".format(e))
+
+    @handled.route("/ok")
+    def ok(request):
+        raise Error("uh oh")
+
+    @nothandled.route("/notok")
+    def notok(request):
+        raise Error("uh oh")
+
+    app.blueprint(handled)
+    app.blueprint(nothandled)
+
+    _, response = app.test_client.get("/ok")
+    assert response.status == 200
+    assert response.text == "handled uh oh"
+
+    _, response = app.test_client.get("/notok")
+    assert response.status == 500
+
+
+def test_bp_exception_handler_not_applied(app):
+    class Error(Exception):
+        pass
+
+    handled = Blueprint("handled")
+    nothandled = Blueprint("nothandled")
+
+    @handled.exception(Error)
+    def handle_error(req, e):
+        return text("handled {}".format(e))
+
+    @nothandled.route("/notok")
+    def notok(request):
+        raise Error("uh oh")
+
+    app.blueprint(handled)
+    app.blueprint(nothandled)
+
+    _, response = app.test_client.get("/notok")
+    assert response.status == 500
+
+
 def test_bp_listeners(app):
     app.route("/")(lambda x: x)
     blueprint = Blueprint("test_middleware")
@@ -782,7 +832,7 @@ def test_static_blueprint_name(static_file_directory, file_name):
 
 @pytest.mark.parametrize("file_name", ["test.file"])
 def test_static_blueprintp_mw(app: Sanic, static_file_directory, file_name):
-    current_file = inspect.getfile(inspect.currentframe())
+    current_file = inspect.getfile(inspect.currentframe())  # type: ignore
     with open(current_file, "rb") as file:
         file.read()
 
@@ -811,31 +861,6 @@ def test_static_blueprintp_mw(app: Sanic, static_file_directory, file_name):
     assert triggered is True
 
 
-def test_route_handler_add(app: Sanic):
-    view = CompositionView()
-
-    async def get_handler(request):
-        return json({"response": "OK"})
-
-    view.add(["GET"], get_handler, stream=False)
-
-    async def default_handler(request):
-        return text("OK")
-
-    bp = Blueprint(name="handler", url_prefix="/handler")
-    bp.add_route(default_handler, uri="/default/", strict_slashes=True)
-
-    bp.add_route(view, uri="/view", name="test")
-
-    app.blueprint(bp)
-
-    _, response = app.test_client.get("/handler/default/")
-    assert response.text == "OK"
-
-    _, response = app.test_client.get("/handler/view")
-    assert response.json["response"] == "OK"
-
-
 def test_websocket_route(app: Sanic):
     event = asyncio.Event()
 
@@ -1028,12 +1053,37 @@ def test_blueprint_registered_multiple_apps():
 
 def test_bp_set_attribute_warning():
     bp = Blueprint("bp")
-    with pytest.warns(UserWarning) as record:
+    message = (
+        "Setting variables on Blueprint instances is not allowed. You should "
+        "change your Blueprint instance to use instance.ctx.foo instead."
+    )
+    with pytest.raises(AttributeError, match=message):
         bp.foo = 1
 
-    assert len(record) == 1
-    assert record[0].message.args[0] == (
-        "Setting variables on Blueprint instances is deprecated "
-        "and will be removed in version 21.9. You should change your "
-        "Blueprint instance to use instance.ctx.foo instead."
-    )
+
+def test_early_registration(app):
+    assert len(app.router.routes) == 0
+
+    bp = Blueprint("bp")
+
+    @bp.get("/one")
+    async def one(_):
+        return text("one")
+
+    app.blueprint(bp)
+
+    assert len(app.router.routes) == 1
+
+    @bp.get("/two")
+    async def two(_):
+        return text("two")
+
+    @bp.get("/three")
+    async def three(_):
+        return text("three")
+
+    assert len(app.router.routes) == 3
+
+    for path in ("one", "two", "three"):
+        _, response = app.test_client.get(f"/{path}")
+        assert response.text == path
