@@ -1,19 +1,15 @@
 import logging
-import os
-import sys
 import uuid
 
 from importlib import reload
 from io import StringIO
+from unittest.mock import Mock
 
 import pytest
 
-from sanic_testing.testing import SanicTestClient
-
 import sanic
 
 from sanic import Sanic
-from sanic.compat import OS_IS_WINDOWS
 from sanic.log import LOGGING_CONFIG_DEFAULTS, logger
 from sanic.response import text
 
@@ -51,7 +47,7 @@ def test_log(app):
 
 def test_logging_defaults():
     # reset_logging()
-    app = Sanic("test_logging")
+    Sanic("test_logging")
 
     for fmt in [h.formatter for h in logging.getLogger("sanic.root").handlers]:
         assert (
@@ -87,7 +83,7 @@ def test_logging_pass_customer_logconfig():
         "format"
     ] = "%(asctime)s - (%(name)s)[%(levelname)s]: %(message)s"
 
-    app = Sanic("test_logging", log_config=modified_config)
+    Sanic("test_logging", log_config=modified_config)
 
     for fmt in [h.formatter for h in logging.getLogger("sanic.root").handlers]:
         assert fmt._fmt == modified_config["formatters"]["generic"]["format"]
@@ -111,11 +107,13 @@ def test_logging_pass_customer_logconfig():
     ),
 )
 def test_log_connection_lost(app, debug, monkeypatch):
-    """ Should not log Connection lost exception on non debug """
+    """Should not log Connection lost exception on non debug"""
     stream = StringIO()
-    root = logging.getLogger("sanic.root")
-    root.addHandler(logging.StreamHandler(stream))
-    monkeypatch.setattr(sanic.server, "logger", root)
+    error = logging.getLogger("sanic.error")
+    error.addHandler(logging.StreamHandler(stream))
+    monkeypatch.setattr(
+        sanic.server.protocols.http_protocol, "error_logger", error
+    )
 
     @app.route("/conn_lost")
     async def conn_lost(request):
@@ -152,62 +150,62 @@ async def test_logger(caplog):
     assert record in caplog.record_tuples
 
 
-@pytest.mark.skipif(
-    OS_IS_WINDOWS and sys.version_info >= (3, 8),
-    reason="Not testable with current client",
-)
-def test_logger_static_and_secure(caplog):
-    # Same as test_logger, except for more coverage:
-    # - test_client initialised separately for static port
-    # - using ssl
-    rand_string = str(uuid.uuid4())
-
-    app = Sanic(name=__name__)
-
-    @app.get("/")
-    def log_info(request):
-        logger.info(rand_string)
-        return text("hello")
-
-    current_dir = os.path.dirname(os.path.realpath(__file__))
-    ssl_cert = os.path.join(current_dir, "certs/selfsigned.cert")
-    ssl_key = os.path.join(current_dir, "certs/selfsigned.key")
-
-    ssl_dict = {"cert": ssl_cert, "key": ssl_key}
-
-    test_client = SanicTestClient(app, port=42101)
-    with caplog.at_level(logging.INFO):
-        request, response = test_client.get(
-            f"https://127.0.0.1:{test_client.port}/",
-            server_kwargs=dict(ssl=ssl_dict),
-        )
-
-    port = test_client.port
-
-    assert caplog.record_tuples[0] == (
-        "sanic.root",
-        logging.INFO,
-        f"Goin' Fast @ https://127.0.0.1:{port}",
-    )
-    assert caplog.record_tuples[1] == (
-        "sanic.root",
-        logging.INFO,
-        f"https://127.0.0.1:{port}/",
-    )
-    assert caplog.record_tuples[2] == ("sanic.root", logging.INFO, rand_string)
-    assert caplog.record_tuples[-1] == (
-        "sanic.root",
-        logging.INFO,
-        "Server Stopped",
-    )
-
-
 def test_logging_modified_root_logger_config():
     # reset_logging()
 
     modified_config = LOGGING_CONFIG_DEFAULTS
     modified_config["loggers"]["sanic.root"]["level"] = "DEBUG"
 
-    app = Sanic("test_logging", log_config=modified_config)
+    Sanic("test_logging", log_config=modified_config)
 
     assert logging.getLogger("sanic.root").getEffectiveLevel() == logging.DEBUG
+
+
+def test_access_log_client_ip_remote_addr(monkeypatch):
+    access = Mock()
+    monkeypatch.setattr(sanic.http, "access_logger", access)
+
+    app = Sanic("test_logging")
+    app.config.PROXIES_COUNT = 2
+
+    @app.route("/")
+    async def handler(request):
+        return text(request.remote_addr)
+
+    headers = {"X-Forwarded-For": "1.1.1.1, 2.2.2.2"}
+
+    request, response = app.test_client.get("/", headers=headers)
+
+    assert request.remote_addr == "1.1.1.1"
+    access.info.assert_called_with(
+        "",
+        extra={
+            "status": 200,
+            "byte": len(response.content),
+            "host": f"{request.remote_addr}:{request.port}",
+            "request": f"GET {request.scheme}://{request.host}/",
+        },
+    )
+
+
+def test_access_log_client_ip_reqip(monkeypatch):
+    access = Mock()
+    monkeypatch.setattr(sanic.http, "access_logger", access)
+
+    app = Sanic("test_logging")
+
+    @app.route("/")
+    async def handler(request):
+        return text(request.ip)
+
+    request, response = app.test_client.get("/")
+
+    access.info.assert_called_with(
+        "",
+        extra={
+            "status": 200,
+            "byte": len(response.content),
+            "host": f"{request.ip}:{request.port}",
+            "request": f"GET {request.scheme}://{request.host}/",
+        },
+    )
