@@ -1,8 +1,6 @@
 import asyncio
 import re
 
-from unittest.mock import Mock
-
 import pytest
 
 from sanic_routing.exceptions import (
@@ -16,7 +14,7 @@ from sanic import Blueprint, Sanic
 from sanic.constants import HTTP_METHODS
 from sanic.exceptions import NotFound, SanicException
 from sanic.request import Request
-from sanic.response import json, text
+from sanic.response import empty, json, text
 
 
 @pytest.mark.parametrize(
@@ -256,9 +254,9 @@ def test_route_strict_slash(app):
 
 
 def test_route_invalid_parameter_syntax(app):
-    with pytest.raises(ValueError):
+    with pytest.raises(InvalidUsage):
 
-        @app.get("/get/<:string>", strict_slashes=True)
+        @app.get("/get/<:str>", strict_slashes=True)
         def handler(request):
             return text("OK")
 
@@ -478,7 +476,7 @@ def test_dynamic_route(app):
 def test_dynamic_route_string(app):
     results = []
 
-    @app.route("/folder/<name:string>")
+    @app.route("/folder/<name:str>")
     async def handler(request, name):
         results.append(name)
         return text("OK")
@@ -513,7 +511,7 @@ def test_dynamic_route_int(app):
 def test_dynamic_route_number(app):
     results = []
 
-    @app.route("/weight/<weight:number>")
+    @app.route("/weight/<weight:float>")
     async def handler(request, weight):
         results.append(weight)
         return text("OK")
@@ -543,9 +541,6 @@ def test_dynamic_route_regex(app):
     async def handler(request, folder_id):
         return text("OK")
 
-    app.router.finalize()
-    print(app.router.find_route_src)
-
     request, response = app.test_client.get("/folder/test")
     assert response.status == 200
 
@@ -587,6 +582,8 @@ def test_dynamic_route_path(app):
     async def handler(request, path):
         return text("OK")
 
+    app.router.finalize()
+
     request, response = app.test_client.get("/path/1/info")
     assert response.status == 200
 
@@ -655,41 +652,46 @@ def test_websocket_route_invalid_handler(app):
 @pytest.mark.asyncio
 @pytest.mark.parametrize("url", ["/ws", "ws"])
 async def test_websocket_route_asgi(app, url):
-    ev = asyncio.Event()
+    @app.after_server_start
+    async def setup_ev(app, _):
+        app.ctx.ev = asyncio.Event()
 
     @app.websocket(url)
     async def handler(request, ws):
-        ev.set()
+        request.app.ctx.ev.set()
 
-    request, response = await app.asgi_client.websocket(url)
-    assert ev.is_set()
+    @app.get("/ev")
+    async def check(request):
+        return json({"set": request.app.ctx.ev.is_set()})
+
+    _, response = await app.asgi_client.websocket(url)
+    _, response = await app.asgi_client.get("/")
+    assert response.json["set"]
 
 
-def test_websocket_route_with_subprotocols(app):
+@pytest.mark.parametrize(
+    "subprotocols,expected",
+    (
+        (["one"], "one"),
+        (["three", "one"], "one"),
+        (["tree"], None),
+        (None, None),
+    ),
+)
+def test_websocket_route_with_subprotocols(app, subprotocols, expected):
     results = []
 
-    @app.websocket("/ws", subprotocols=["foo", "bar"])
+    @app.websocket("/ws", subprotocols=["zero", "one", "two", "three"])
     async def handler(request, ws):
-        results.append(ws.subprotocol)
+        nonlocal results
+        results = ws.subprotocol
         assert ws.subprotocol is not None
 
-    _, response = SanicTestClient(app).websocket("/ws", subprotocols=["bar"])
-    assert response.opened is True
-    assert results == ["bar"]
-
     _, response = SanicTestClient(app).websocket(
-        "/ws", subprotocols=["bar", "foo"]
+        "/ws", subprotocols=subprotocols
     )
     assert response.opened is True
-    assert results == ["bar", "bar"]
-
-    _, response = SanicTestClient(app).websocket("/ws", subprotocols=["baz"])
-    assert response.opened is True
-    assert results == ["bar", "bar", None]
-
-    _, response = SanicTestClient(app).websocket("/ws")
-    assert response.opened is True
-    assert results == ["bar", "bar", None, None]
+    assert results == expected
 
 
 @pytest.mark.parametrize("strict_slashes", [True, False, None])
@@ -824,7 +826,7 @@ def test_dynamic_add_route_string(app):
         results.append(name)
         return text("OK")
 
-    app.add_route(handler, "/folder/<name:string>")
+    app.add_route(handler, "/folder/<name:str>")
     request, response = app.test_client.get("/folder/test123")
 
     assert response.text == "OK"
@@ -860,7 +862,7 @@ def test_dynamic_add_route_number(app):
         results.append(weight)
         return text("OK")
 
-    app.add_route(handler, "/weight/<weight:number>")
+    app.add_route(handler, "/weight/<weight:float>")
 
     request, response = app.test_client.get("/weight/12345")
     assert response.text == "OK"
@@ -1008,14 +1010,8 @@ def test_unmergeable_overload_routes(app):
     async def handler2(request):
         return text("OK1")
 
-    assert (
-        len(
-            dict(list(app.router.static_routes.values())[0].handlers)[
-                "overload_whole"
-            ]
-        )
-        == 3
-    )
+    assert len(app.router.static_routes) == 1
+    assert len(app.router.static_routes[("overload_whole",)].methods) == 3
 
     request, response = app.test_client.get("/overload_whole")
     assert response.text == "OK1"
@@ -1073,7 +1069,8 @@ def test_uri_with_different_method_and_different_params(app):
         return json({"action": action})
 
     request, response = app.test_client.get("/ads/1234")
-    assert response.status == 405
+    assert response.status == 200
+    assert response.json == {"ad_id": "1234"}
 
     request, response = app.test_client.post("/ads/post")
     assert response.status == 200
@@ -1231,3 +1228,41 @@ def test_routes_with_and_without_slash_definitions(app):
         _, response = app.test_client.post(f"/{term}/")
         assert response.status == 200
         assert response.text == f"{term}_with"
+
+
+def test_added_route_ctx_kwargs(app):
+    @app.route("/", ctx_foo="foo", ctx_bar=99)
+    async def handler(request: Request):
+        return empty()
+
+    request, _ = app.test_client.get("/")
+
+    assert request.route.ctx.foo == "foo"
+    assert request.route.ctx.bar == 99
+
+
+def test_added_bad_route_kwargs(app):
+    message = "Unexpected keyword arguments: foo, bar"
+    with pytest.raises(TypeError, match=message):
+
+        @app.route("/", foo="foo", bar=99)
+        async def handler(request: Request):
+            ...
+
+
+@pytest.mark.asyncio
+async def test_added_callable_route_ctx_kwargs(app):
+    def foo(*args, **kwargs):
+        return "foo"
+
+    async def bar(*args, **kwargs):
+        return 99
+
+    @app.route("/", ctx_foo=foo, ctx_bar=bar)
+    async def handler(request: Request):
+        return empty()
+
+    request, _ = await app.asgi_client.get("/")
+
+    assert request.route.ctx.foo() == "foo"
+    assert await request.route.ctx.bar() == 99
