@@ -8,7 +8,7 @@ import pytest
 
 from sanic_testing.testing import HOST, PORT
 
-from sanic.exceptions import InvalidUsage
+from sanic.exceptions import InvalidUsage, SanicException
 
 
 AVAILABLE_LISTENERS = [
@@ -33,9 +33,17 @@ def create_listener(listener_name, in_list):
     return _listener
 
 
+def create_listener_no_loop(listener_name, in_list):
+    async def _listener(app):
+        print(f"DEBUG MESSAGE FOR PYTEST for {listener_name}")
+        in_list.insert(0, app.name + listener_name)
+
+    return _listener
+
+
 def start_stop_app(random_name_app, **run_kwargs):
     def stop_on_alarm(signum, frame):
-        raise KeyboardInterrupt("SIGINT for sanic to stop gracefully")
+        random_name_app.stop()
 
     signal.signal(signal.SIGALRM, stop_on_alarm)
     signal.alarm(1)
@@ -56,6 +64,17 @@ def test_single_listener(app, listener_name):
     assert app.name + listener_name == output.pop()
 
 
+@skipif_no_alarm
+@pytest.mark.parametrize("listener_name", AVAILABLE_LISTENERS)
+def test_single_listener_no_loop(app, listener_name):
+    """Test that listeners on their own work"""
+    output = []
+    # Register listener
+    app.listener(listener_name)(create_listener_no_loop(listener_name, output))
+    start_stop_app(app)
+    assert app.name + listener_name == output.pop()
+
+
 @skipif_no_alarm
 @pytest.mark.parametrize("listener_name", AVAILABLE_LISTENERS)
 def test_register_listener(app, listener_name):
@@ -101,12 +120,16 @@ async def test_trigger_before_events_create_server(app):
 
     @app.listener("before_server_start")
     async def init_db(app, loop):
-        app.db = MySanicDb()
+        app.ctx.db = MySanicDb()
 
-    await app.create_server(debug=True, return_asyncio_server=True, port=PORT)
+    srv = await app.create_server(
+        debug=True, return_asyncio_server=True, port=PORT
+    )
+    await srv.startup()
+    await srv.before_start()
 
-    assert hasattr(app, "db")
-    assert isinstance(app.db, MySanicDb)
+    assert hasattr(app.ctx, "db")
+    assert isinstance(app.ctx.db, MySanicDb)
 
 
 @pytest.mark.asyncio
@@ -118,14 +141,17 @@ async def test_trigger_before_events_create_server_missing_event(app):
 
         @app.listener
         async def init_db(app, loop):
-            app.db = MySanicDb()
+            app.ctx.db = MySanicDb()
 
-    assert not hasattr(app, "db")
+    assert not hasattr(app.ctx, "db")
 
 
 def test_create_server_trigger_events(app):
     """Test if create_server can trigger server events"""
 
+    def stop_on_alarm(signum, frame):
+        raise KeyboardInterrupt("...")
+
     flag1 = False
     flag2 = False
     flag3 = False
@@ -133,8 +159,7 @@ def test_create_server_trigger_events(app):
     async def stop(app, loop):
         nonlocal flag1
         flag1 = True
-        await asyncio.sleep(0.1)
-        app.stop()
+        signal.alarm(1)
 
     async def before_stop(app, loop):
         nonlocal flag2
@@ -151,20 +176,23 @@ def test_create_server_trigger_events(app):
     loop = asyncio.get_event_loop()
 
     # Use random port for tests
+
+    signal.signal(signal.SIGALRM, stop_on_alarm)
     with closing(socket()) as sock:
         sock.bind(("127.0.0.1", 0))
 
         serv_coro = app.create_server(return_asyncio_server=True, sock=sock)
         serv_task = asyncio.ensure_future(serv_coro, loop=loop)
         server = loop.run_until_complete(serv_task)
-        server.after_start()
+        loop.run_until_complete(server.startup())
+        loop.run_until_complete(server.after_start())
         try:
             loop.run_forever()
-        except KeyboardInterrupt as e:
+        except KeyboardInterrupt:
             loop.stop()
         finally:
             # Run the on_stop function if provided
-            server.before_stop()
+            loop.run_until_complete(server.before_stop())
 
             # Wait for server to close
             close_task = server.close()
@@ -174,5 +202,32 @@ def test_create_server_trigger_events(app):
             signal.stopped = True
             for connection in server.connections:
                 connection.close_if_idle()
-            server.after_stop()
+            loop.run_until_complete(server.after_stop())
         assert flag1 and flag2 and flag3
+
+
+@pytest.mark.asyncio
+async def test_missing_startup_raises_exception(app):
+    @app.listener("before_server_start")
+    async def init_db(app, loop):
+        ...
+
+    srv = await app.create_server(
+        debug=True, return_asyncio_server=True, port=PORT
+    )
+
+    with pytest.raises(SanicException):
+        await srv.before_start()
+
+
+def test_reload_listeners_attached(app):
+    async def dummy(*_):
+        ...
+
+    app.reload_process_start(dummy)
+    app.reload_process_stop(dummy)
+    app.listener("reload_process_start")(dummy)
+    app.listener("reload_process_stop")(dummy)
+
+    assert len(app.listeners.get("reload_process_start")) == 2
+    assert len(app.listeners.get("reload_process_stop")) == 2
